package zero1hd.polyjet.audio.map;

import com.badlogic.gdx.audio.Music;
import com.badlogic.gdx.utils.Array;

import zero1hd.polyjet.audio.AudioData;
import zero1hd.polyjet.entity.Entities;

public class GamePlayMap {
	private Music playableClip;
	private float playTime;
	private Array<EntitySpawnInfo> spawnList;
	private boolean building;
	private int index;
	
	/**
	 * GamePlayMap is what the game area will use to generate entities and judge current audio data
	 * @param audioData audio data
	 */
	public GamePlayMap(AudioData audioData) {
		playableClip = audioData.getPlaybackMusic();
		playTime = audioData.getDuration();
		spawnList = new Array<>();
	}
	
	/**
	 * returns audio data
	 * @return
	 */
	public Music getPlayableClip() {
		return playableClip;
	}
	
	/**
	 * returns the runtime time of the overall song
	 * @return
	 */
	public float getPlayTime() {
		return playTime;
	}

	/**
	 * call this when beginning to build map
	 */
	public void beginBuild() {
		building = true;
	}

	/**
	 * call this when finishing building map
	 */
	public void endBuild() {
		building = false;
	}

	
	/**
	 * used to add data types for what entity to spawn at that index position
	 * cannot be null.
	 * @param entityType what type of entity to spawn
	 * @param parameters the arguments for the entity. It is important to have the same amount of parameters the entity requires to spawn
	 */
	public void addToMap(Entities entityType, double... parameters) {
		if (building && entityType != null && parameters != null) {
			spawnList.add(new EntitySpawnInfo(entityType, parameters));
		}
	}
	
	/**
	 * use this to add rest for current index position
	 */
	public void addRestToMap() {
		if (building) {
			spawnList.add(null);
		}
	}
	
	/**
	 * Only works when not started.
	 * retrieve next entity in list.
	 * @return
	 */
	public EntitySpawnInfo nextEntity() {
		if (!building) {
			index++;
			
			EntitySpawnInfo spawnInfo = spawnList.get(index);
			return spawnInfo;
		} else {
			return null;
		}
	}
	
	/**
	 * set retrieve entity index position.
	 * @param index
	 */
	public void setIndex(int index) {
		this.index = index;
	}
}
