package zero1hd.polyjet.audio.map;

import org.apache.commons.math3.random.MersenneTwister;

import com.badlogic.gdx.math.MathUtils;
import com.badlogic.gdx.utils.FloatArray;

import zero1hd.polyjet.Polyjet;
import zero1hd.polyjet.audio.AudioAnalyzer;
import zero1hd.polyjet.entity.Entities;

public class RhythmMapAlgorithm implements Runnable {
	private FloatArray bassPeaks;
	private FloatArray UMPeaks;
	private FloatArray overlappedPeaks;
	private MersenneTwister rand;
	private GamePlayMap map;
	private float avgBPS;
	
	private float speedMod, healthMod;
	
	private float windowPerSecond;
	
	private volatile int progress;
	public RhythmMapAlgorithm(AudioAnalyzer analyzer, float speedMod, float healthMod) {
		bassPeaks = analyzer.getBassPeaks();
		UMPeaks = analyzer.getUMPeaks();
		overlappedPeaks = analyzer.getOverlappedPeaks();
		map = new GamePlayMap(analyzer.getAudioData());
		rand = new MersenneTwister(analyzer.getPUID());
		avgBPS = analyzer.getAvgBPS();
		windowPerSecond = 1/analyzer.getsecondsPerWindow();
		
		this.speedMod = speedMod;
		this.healthMod = healthMod;
	}
	
	@Override
	public void run() {
		map.beginBuild();
		for (int index = 0; index < bassPeaks.size; index++) {
			if (bassPeaks.get(index) != 0 || UMPeaks.get(index) != 0) {
				int warningTime = (int) ((3/speedMod)*windowPerSecond);
				if (overlappedPeaks.get(index + warningTime) != 0) {
					//TODO basic void circle spawning
					float endRadius = overlappedPeaks.get(index+3)*Polyjet.GAME_AREA_WIDTH;
					
					map.addToMap(Entities.VOID_CIRCLE, 
							endRadius, 
							rand.nextFloat()*Polyjet.GAME_AREA_WIDTH, 
							rand.nextFloat()*Polyjet.GAME_AREA_HEIGHT,
							endRadius/avgBPS,
							warningTime
							);
				}
				if (bassPeaks.get(index) != 0) {
					map.addToMap(Entities.BAR, 
							MathUtils.round(rand.nextFloat()*Polyjet.GAME_AREA_WIDTH), 
							((Polyjet.GAME_AREA_HEIGHT-bassPeaks.get(index)*Polyjet.GAME_AREA_HEIGHT)/avgBPS)*speedMod);
				} else {
					if (UMPeaks.get(index) != 0) {
						//TODO basic pellet scatter spawn
					}
					//TODO rest of the basic generation
				}
			} else {
				map.addRestToMap();
			}
			
			progress = (int) (100f*index/bassPeaks.size);
		}
		map.endBuild();
	}
	
	public synchronized GamePlayMap getMap() {
		return map;
	}
	
	public synchronized int getProgress() {
		return progress;
	}
}
